#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
#
# Copyright 2008 Sun Microsystems, Inc. All rights reserved.
#
# The contents of this file are subject to the terms of either the GNU
# General Public License Version 2 only ("GPL") or the Common Development
# and Distribution License("CDDL") (collectively, the "License").  You
# may not use this file except in compliance with the License. You can obtain
# a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
# or updatetool/LICENSE.txt.  See the License for the specific
# language governing permissions and limitations under the License.
#
# When distributing the software, include this License Header Notice in each
# file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
# Sun designates this particular file as subject to the "Classpath" exception
# as provided by Sun in the GPL Version 2 section of the License file that
# accompanied this code.  If applicable, add the following below the License
# Header, with the fields enclosed by brackets [] replaced by your own
# identifying information: "Portions Copyrighted [year]
# [name of copyright owner]"
#
# Contributor(s):
#
# If you wish your version of this file to be governed by only the CDDL or
# only the GPL Version 2, indicate your decision by adding "[Contributor]
# elects to include this software in this distribution under the [CDDL or GPL
# Version 2] license."  If you don't indicate a single choice of license, a
# recipient has the option to distribute your version of this file under
# either the CDDL, the GPL Version 2 or to extend the choice of license to
# its licensees as provided above.  However, if you add GPL Version 2 code
# and therefore, elected the GPL Version 2 license, then the option applies
# only if the new code is made subject to such option by the copyright
# holder.
#

# copypkgs - script for copying IPS packages from one repo to another (file system based)

import os
import sys
import traceback
import getopt
import shutil

def usage(msg):
    ret = """Usage: %s -s src-repo-dir -d dest-repo-dir pkg-names...
Options:
    -s src-repo-dir    repository directory to copy packages from
    -d dest-repo-dir   repository directory to copy package to
    """ % (__file__,)
    if msg is not None:
        ret = "%s\n\n%s" % (msg, ret)

    print >> sys.stderr, ret

def hash_file_name(f):
    """Return the two-level path fragment for the given filename, which is
    assumed to be a content hash of at least 8 distinct characters."""
    return os.path.join("%s" % f[0:2], "%s" % f[2:8], "%s" % f)

def main_func():
    try:
        opts, pargs = getopt.getopt(sys.argv[1:], "s:d:")
    except getopt.GetoptError, e:
        usage("copypkgs: illegal option -- %s" % e.opt)
        return 1

    srcrepo = None
    destrepo = None
    for opt, arg in opts:
        if opt == "-s":
            srcrepo = arg
        if opt == "-d":
            destrepo = arg

    if srcrepo == None or destrepo == None:
        usage("invalid arguments: both -s and -d must be specified")
        return 1

    if len(pargs) == 0:
        usage("invalid arguments: no package names listed")
        return 1

    pkgs = pargs

    print "source directory:", srcrepo
    print "destination directory:", destrepo
    print "package list:", pkgs

    srcfdir = os.path.join(srcrepo, "file");
    destfdir = os.path.join(destrepo, "file");
    
    for p in pkgs:
        srcpdir = os.path.join(srcrepo, "pkg", p)
        destpdir = os.path.join(destrepo, "pkg", p)
        # copy all versions
        for v in os.listdir(srcpdir):
            print "copying %s@%s" % (p, v),
            
            # copy the manifest
            src_manifest = os.path.join(srcpdir, v)
            dest_manifest = os.path.join(destpdir, v)
            if not os.path.exists(destpdir): 
                os.makedirs(destpdir)
            shutil.copyfile(src_manifest, dest_manifest)
            
            # copy each file that the manifest references
            mcontent = file(src_manifest).read()
            for l in mcontent.splitlines():
                l = l.lstrip()
                if not l or l[0] == "#":
                    continue
                type, l = l.split(" ", 1)
                filetypes = [ "file", "license" ]
                if type not in filetypes:
                    continue
                hash, l = l.split(" ", 1)
                srcpath = os.path.join(srcfdir, hash_file_name(hash))
                destpath = os.path.join(destfdir, hash_file_name(hash))
                sys.stdout.write(".")
                if not os.path.exists(os.path.dirname(destpath)): 
                    os.makedirs(os.path.dirname(destpath))
                shutil.copyfile(srcpath, destpath)
            print

    print "copy complete."
    return 0

if __name__ == "__main__":
    try:
        ret = main_func()
    except SystemExit, e:
        raise e
    except KeyboardInterrupt:
        print "Interrupted"
        sys.exit(1)
    except:
        traceback.print_exc()
        sys.exit(99)
    sys.exit(ret)

